@@ -8,12 +8,15 @@ import java.awt.Graphics;
 import java.awt.Insets;
 import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.event.ActionEvent;
 import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
 import java.awt.event.MouseMotionListener;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.function.Function;
+import javax.swing.Action;
 import javax.swing.BoxLayout;
 import javax.swing.DefaultListCellRenderer;
 import javax.swing.JCheckBox;
@@ -44,12 +47,10 @@ import javax.swing.event.MouseInputAdapter;
  * @author   Mark Taylor
  * @since    13 Mar 2013
  */
-@SuppressWarnings({"unchecked","rawtypes"})
-public abstract class CheckBoxList<T> extends JList {
+public abstract class CheckBoxList<T> extends JList<T> {
 
-    private final Class<T> clazz_;
     private final boolean canSelect_;
-    private final CheckBoxCellRenderer renderer_;
+    private final CheckBoxCellRenderer<?> renderer_;
     private final DragListener dragger_;
     private final List<ListDataListener> listeners_;
     private String[] msgLines_;
@@ -59,16 +60,15 @@ public abstract class CheckBoxList<T> extends JList {
      * A renderer component is supplied; when entries need to be displayed
      * the <code>configureEntryRenderer</code> method is called.
      *
-     * @param   clazz  supertype for each entry in the list
      * @param   model  list model
      * @param   canSelect   true if list item selection is permitted
-     * @param   entryRenderer   renderer for list entry contents
-     *                          (excluding drag and checkbox decorations)
+     * @param   entryRendering   rendering for list entry contents
+     *                           (excluding drag and checkbox decorations)
      */
-    public CheckBoxList( Class<T> clazz, ListModel model, boolean canSelect,
-                         JComponent entryRenderer ) {
+    @SuppressWarnings("this-escape")
+    public CheckBoxList( ListModel<T> model, boolean canSelect,
+                         Rendering<T,?> entryRendering ) {
         super( model );
-        clazz_ = clazz;
         canSelect_ = canSelect;
 
         /* Arrange to forward ListDataEvents from the base list model to
@@ -94,7 +94,7 @@ public abstract class CheckBoxList<T> extends JList {
         } );
 
         /* Set up cell rendering. */
-        renderer_ = new CheckBoxCellRenderer( entryRenderer );
+        renderer_ = createCheckBoxCellRenderer( entryRendering );
         setCellRenderer( renderer_ );
 
         /* Remove any default mouse listeners and replace them with
@@ -116,16 +116,6 @@ public abstract class CheckBoxList<T> extends JList {
         addMouseMotionListener( dragger_ );
     }
 
-    /**
-     * This method is called whenever the list cell needs to be painted.
-     *
-     * @param  entryRenderer  renderer object supplied at construction time
-     * @param  item   list entry
-     * @param  index  index in list at which entry appears
-     */
-    protected abstract void configureEntryRenderer( JComponent entryRenderer,
-                                                    T item, int index );
-
     /**
      * Indicates whether the checkbox for a given item is selected.
      *
@@ -156,19 +146,20 @@ public abstract class CheckBoxList<T> extends JList {
     public abstract void moveItem( int ifrom, int ito );
 
     /**
-     * Returns a list cell entry cast to the entry type of this list,
-     * or null if it can't be done.
+     * Programmatically sets the checkbox status for all the items in the list.
      *
-     * @param  value   list entry
-     * @return  typed list entry, or null
+     * <p>The default implementation calls {@link #setChecked setChecked}
+     * for each item in the list, but subclasses may be able to
+     * override this for improved efficiency.
+     *
+     * @param  isChecked   true to check all, false to uncheck all
      */
-    public T getTypedValue( Object value ) {
-        return clazz_.isInstance( value ) ? clazz_.cast( value ) : null;
-    }
-
-    @Override
-    public T getSelectedValue() {
-        return getTypedValue( super.getSelectedValue() );
+    public void setCheckedAll( boolean isChecked ) {
+        ListModel<T> model = getModel();
+        int nItem = model.getSize();
+        for ( int i = 0; i < nItem; i++ ) {
+            setChecked( model.getElementAt( i ), isChecked );
+        }
     }
 
     /**
@@ -202,6 +193,24 @@ public abstract class CheckBoxList<T> extends JList {
         listeners_.remove( l );
     }
 
+    /**
+     * Returns a new action that can be used to check or uncheck all
+     * the items in the list at once.
+     *
+     * @param  isChecked   true to check all, false to uncheck all
+     */
+    public Action createCheckAllAction( final boolean isChecked ) {
+        String act = isChecked ? "Select" : "Unselect";
+        return new BasicAction( act + " All",
+                                isChecked ? ResourceIcon.REVEAL_ALL
+                                          : ResourceIcon.HIDE_ALL,
+                                act + " all the listed items" ) {
+            public void actionPerformed( ActionEvent evt ) {
+                setCheckedAll( isChecked );
+            }
+        };
+    }
+
     @Override
     protected void paintComponent( Graphics g ) {
         super.paintComponent( g );
@@ -212,8 +221,7 @@ public abstract class CheckBoxList<T> extends JList {
         Point itemBase = indexToLocation( dragger_.fromIndex_ );
         if ( dragItem != null ) {
             int fromIndex = dragger_.fromIndex_;
-            CheckBoxCellRenderer dragComp =
-                (CheckBoxCellRenderer)
+            CheckBoxCellRenderer<?> dragComp =
                 renderer_.getListCellRendererComponent(
                     this, dragItem, fromIndex, isSelectedIndex( fromIndex ),
                     hasFocus() );
@@ -252,6 +260,63 @@ public abstract class CheckBoxList<T> extends JList {
         }
     }
 
+    /**
+     * Invokes CheckBoxCellRenderer constructor taking care of generics.
+     *
+     * @param  rendering   rendering definition
+     * @return  new renderer
+     */
+    private <C extends JComponent> CheckBoxCellRenderer<C>
+            createCheckBoxCellRenderer( Rendering<T,C> rendering ) {
+        return new CheckBoxCellRenderer<C>( rendering );
+    }
+
+    /**
+     * Returns a simple rendering based on a JLabel.
+     *
+     * @param  stringify  function that maps a list item to
+     *                    a string for display
+     * @return   cell rendering
+     */
+    public static <T> Rendering<T,JLabel>
+            createLabelRendering( Function<T,String> stringify ) {
+        return new Rendering<T,JLabel>() {
+            public JLabel createRendererComponent() {
+                return new JLabel();
+            }
+            public void configureRendererComponent( JLabel label,
+                                                    T item, int index ) {
+                label.setText( stringify.apply( item ) );
+            }
+        };
+    }
+
+    /**
+     * Defines the rendering of the contents of a list cell.
+     * This is effectively an easier to use replacement for
+     * {@link javax.swing.ListCellRenderer}.
+     */
+    public static interface Rendering<T,C extends JComponent> {
+
+        /**
+         * Creates a component which can be used for cell rendering.
+         *
+         * @return   renderer component
+         */
+        C createRendererComponent();
+
+        /**
+         * Configures the renderer component's appearance according to
+         * the cell index and content.
+         * This method is called whenever the list cell needs to be painted.
+         *
+         * @param  component   renderer component as supplied by this instance
+         * @param  item   list entry
+         * @param  index  index in list at which entry appears
+         */
+        void configureRendererComponent( C component, T item, int index );
+    }
+
     /**
      * Mouse listener that looks out for clicks on the checkbox part of
      * list cells and behaves as if the checkbox is working.
@@ -274,7 +339,7 @@ public abstract class CheckBoxList<T> extends JList {
             Point point = evt.getPoint();
             if ( isCheckbox( point ) ) {
                 int index = locationToIndex( point );
-                T item = getTypedValue( getModel().getElementAt( index ) );
+                T item = getModel().getElementAt( index );
                 if ( item != null ) {
                     setChecked( item, ! isChecked( item ) );
                     ListDataEvent devt =
@@ -373,7 +438,7 @@ public abstract class CheckBoxList<T> extends JList {
 
         @Override
         public void mouseDragged( MouseEvent evt ) {
-            dragItem_ = getTypedValue( getModel().getElementAt( fromIndex_ ) );
+            dragItem_ = getModel().getElementAt( fromIndex_ );
             dragPoint_ = evt.getPoint();
             CheckBoxList.this.repaint();
         }
@@ -382,10 +447,12 @@ public abstract class CheckBoxList<T> extends JList {
     /**
      * Custom ListCellRenderer implementation for a CheckBoxList.
      */
-    private class CheckBoxCellRenderer extends JPanel
-                                       implements ListCellRenderer {
+    private class CheckBoxCellRenderer<C extends JComponent>
+            extends JPanel
+            implements ListCellRenderer<T> {
 
-        private final JComponent entryRenderer_;
+        private final Rendering<T,C> rendering_;
+        private final C entryRenderer_;
         private final JCheckBox checkBox_;
         private final JLabel handle_;
         private final DefaultListCellRenderer dfltRenderer_;
@@ -393,11 +460,12 @@ public abstract class CheckBoxList<T> extends JList {
         /**
          * Constructor.
          *
-         * @param   entryRenderer   renderer for list entry contents
-         *                          (excluding drag and checkbox decorations)
+         * @param   rendering   defines rendering for list entry contents
+         *                      (excluding drag and checkbox decorations)
          */
-        CheckBoxCellRenderer( JComponent entryRenderer) {
-            entryRenderer_ = entryRenderer;
+        CheckBoxCellRenderer( Rendering<T,C> rendering ) {
+            rendering_ = rendering;
+            entryRenderer_ = rendering.createRendererComponent();
             setLayout( new BoxLayout( this, BoxLayout.X_AXIS ) );
             checkBox_ = new JCheckBox();
             checkBox_.setOpaque( false );
@@ -409,19 +477,21 @@ public abstract class CheckBoxList<T> extends JList {
             add( entryRenderer_ );
         }
 
-        public Component getListCellRendererComponent( JList list, Object value,
-                                                       int index, boolean isSel,
-                                                       boolean hasFocus ) {
-            dfltRenderer_.getListCellRendererComponent( list, value, index,
+        public CheckBoxCellRenderer<C>
+                getListCellRendererComponent( JList<? extends T> list,
+                                              T item, int index,
+                                              boolean isSel,
+                                              boolean hasFocus ) {
+            dfltRenderer_.getListCellRendererComponent( list, item, index,
                                                         isSel, hasFocus );
             setBackground( dfltRenderer_.getBackground() );
             setForeground( dfltRenderer_.getForeground() );
             setBorder( dfltRenderer_.getBorder() );
-            T item = getTypedValue( value );
             final int itemWidth;
             if ( item != null ) {
                 checkBox_.setSelected( isChecked( item ) );
-                configureEntryRenderer( entryRenderer_, item, index );
+                rendering_.configureRendererComponent( entryRenderer_,
+                                                       item, index );
                 entryRenderer_.validate();
                 itemWidth = entryRenderer_.getPreferredSize().width;
             }
