@@ -123,7 +123,9 @@ public class ResourceIcon implements Icon {
         SCROLLER = makeIcon( "scroll2.png" ),
         PARALLEL = makeIcon( "parallel1.png" ),
         STILTS = makeIcon( "stilts.png" ),
+        STILTS_HELP = makeIcon( "stiltshelp.png" ),
         EDIT = makeIcon( "pencil.png" ),
+        CLIPBOARD = makeIcon( "clipboard.png" ),
 
         /* Windows. */
         CONTROL = makeIcon( "controlw.png" ),
