@@ -1,6 +1,7 @@
 package uk.ac.starlink.topcat;
 
 import gnu.jel.CompilationException;
+import gnu.jel.CompiledExpression;
 import java.awt.Component;
 import java.awt.Toolkit;
 import java.awt.Window;
@@ -9,6 +10,7 @@ import java.awt.event.ActionListener;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Comparator;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.Iterator;
@@ -51,6 +53,8 @@ import uk.ac.starlink.topcat.activate.ActivationMeta;
 import uk.ac.starlink.topcat.activate.ActivationWindow;
 import uk.ac.starlink.ttools.convert.Conversions;
 import uk.ac.starlink.ttools.convert.ValueConverter;
+import uk.ac.starlink.ttools.jel.JELUtils;
+import uk.ac.starlink.util.gui.ErrorDialog;
 
 /**
  * Defines all the state for the representation of a given table as
@@ -76,12 +80,14 @@ public class TopcatModel {
     private final OptionsListModel<RowSubset> subsets_;
     private final Map<RowSubset,Long> subsetCounts_;
     private final SingleRowSubset activatedSubset_;
-    private final ComboBoxModel<SortOrder> sortSelectionModel_;
+    private final SortOrderSelector.Model sortSelectionModel_;
     private final ComboBoxModel<RowSubset> subsetSelectionModel_;
-    private final SortSenseModel sortSenseModel_;
+    private final JToggleButton.ToggleButtonModel sortSenseModel_;
     private final Collection<TopcatListener> listeners_;
     private final Map<ValueInfo,ColumnSelectorModel> columnSelectorMap_;
     private final TableBuilder tableBuilder_;
+    private SortOrder sortOrder_;
+    private boolean sortSense_;
     private ActivationWindow activationWindow_;
     private SubsetConsumerDialog subsetConsumerDialog_;
     private final int id_;
@@ -107,6 +113,7 @@ public class TopcatModel {
      * @param   location  location string
      * @param   controlWindow  control window instance
      */
+    @SuppressWarnings("this-escape")
     protected TopcatModel( StarTable startab, int id, String location,
                            ControlWindow controlWindow ) {
         controlWindow_ = controlWindow;
@@ -179,8 +186,11 @@ public class TopcatModel {
         columnList_ = new ColumnList( columnModel_ );
 
         /* Set up the current sort selector. */
-        sortSelectionModel_ = new SortSelectionModel();
-        sortSenseModel_ = new SortSenseModel();
+        sortSenseModel_ = new JToggleButton.ToggleButtonModel();
+        sortSelectionModel_ = new SortOrderSelector.Model( this );
+        sortSenseModel_.setSelected( true );
+        sortSense_ = sortSenseModel_.isSelected();
+        sortOrder_ = sortSelectionModel_.getSelectedSortOrder();
 
         /* Initialise subsets list. */
         activatedSubset_ = new SingleRowSubset( "Activated" );
@@ -248,7 +258,7 @@ public class TopcatModel {
     }
 
     /**
-     * Retursn the model's ID number.  This is a small sequence number, 
+     * Returns the model's ID number.  This is a small sequence number, 
      * typically starting
      * at one and increasing for every model created in this topcat instance.
      * It's used for identifying the model to the user.
@@ -372,7 +382,7 @@ public class TopcatModel {
      *
      * @return sort selection model
      */
-    public ComboBoxModel<SortOrder> getSortSelectionModel() {
+    public SortOrderSelector.Model getSortSelectionModel() {
         return sortSelectionModel_;
     }
 
@@ -433,7 +443,7 @@ public class TopcatModel {
      * @return  current sort order
      */
     public SortOrder getSelectedSort() {
-        return (SortOrder) sortSelectionModel_.getSelectedItem();
+        return sortSelectionModel_.getSelectedSortOrder();
     }
 
     /**
@@ -588,11 +598,10 @@ public class TopcatModel {
      */
     public Action getSortAction( final SortOrder order, 
                                  final boolean ascending ) {
-        TableColumn tcol = order.getColumn();
         return new BasicAction( "Sort " + ( ascending ? "up" : "down" ),
                                 ascending ? ResourceIcon.UP : ResourceIcon.DOWN,
                                 "Sort rows by " + ( ascending ? "a" : "de" ) +
-                                "scending order of " + tcol.getIdentifier() ) {
+                                "scending order of " + order ) {
             public void actionPerformed( ActionEvent evt ) {
                 sortBy( order, ascending );
             }
@@ -927,17 +936,62 @@ public class TopcatModel {
 
     /**
      * Trigger a sort of the rows in the viewModel.
-     * This causes a {@link TopcatEvent#CURRENT_ORDER} event to be sent
-     * to listeners.
+     * In case of a material change this causes a
+     * {@link TopcatEvent#CURRENT_ORDER} event to be sent to listeners.
      *
      * @param  order  sort order
      * @param  ascending  sort sense (true for up, false for down)
      */
     public void sortBy( SortOrder order, boolean ascending ) {
-        if ( order != SortOrder.NONE ) {
-            sortSenseModel_.setSelected( ascending );
+
+        /* Transient? */
+        if ( order == null ) {
+            assert false;
+            return;
         }
-        sortSelectionModel_.setSelectedItem( order );
+
+        /* No change. */
+        else if ( order.equals( sortOrder_ ) && ascending == sortSense_ ) {
+            return;
+        }
+
+        /* There is a material change.  Recalculate the rowMap array. */
+        int[] rowMap;
+        if ( order.equals( SortOrder.NONE ) ) {
+            rowMap = null;
+        }
+        else if ( order.equals( sortOrder_ ) ) {
+
+            /* If only the sense has changed, reverse the row map
+             * in place. */
+            assert ascending != sortSense_;
+                rowMap = viewModel_.getRowMap();
+                for ( int i = 0, j = rowMap.length - 1; i < j; i++, j-- ) {
+                    int c = rowMap[ i ];
+                    rowMap[ i ] = rowMap[ j ];
+                    rowMap[ j ] = c;
+                }
+        }
+        else {
+            try {
+                rowMap = getSortedRowMap( order, ascending );
+            }
+            catch ( CompilationException | IOException e ) {
+                ErrorDialog.showError( ControlWindow.getInstance(),
+                                       "Sort Error", e );
+                return;
+            }
+        }
+
+        /* Store the changed state. */
+        sortSense_ = ascending;
+        sortOrder_ = order;
+        viewModel_.setOrder( rowMap );
+        sortSenseModel_.setSelected( ascending );
+        sortSelectionModel_.setSelectedSortOrder( order );
+
+        /* Inform listeners. */
+        fireModelChanged( TopcatEvent.CURRENT_ORDER, null );
     }
 
     /**
@@ -967,59 +1021,71 @@ public class TopcatModel {
      * Returns a row mapping array which gives the sort order corresponding
      * to a sort on values in a given column.
      * 
-     * @param  icol  the index of the column to be sorted on in
-     *               this viewer's model  
-     * @param  ascending  true for ascending sort, false for descending
+     * @param  order  sort order for the table
+     * @param  isAscending  true for ascending sort, false for descending
      */
-    private int[] getSortOrder( int icol, final boolean ascending )
-            throws IOException { 
-        final int sense = ascending ? 1 : -1;
-         
-        /* Define a little class for objects being sorted. */
-        class Item implements Comparable<Item> {
-            final int rank;
-            final Comparable<?> value;
-            Item( int r, Comparable<?> v ) {
-                this.rank = r;
-                this.value = v;
-            }
-            public int compareTo( Item other ) {
-                Comparable<?> oval = other.value;
-                if ( value != null && oval != null ) {
-                    @SuppressWarnings("unchecked")
-                    int cmp = ((Comparable<Object>) value).compareTo( oval ); 
-                    return sense * cmp;
-                } 
-                else if ( value == null && oval == null ) {
-                    return 0;
-                }
-                else {
-                    return sense * ( ( value == null ) ? 1 : -1 );
-                }
-            }
-        }
+    private int[] getSortedRowMap( SortOrder order, boolean isAscending )
+            throws IOException, CompilationException {
+        final int sense = isAscending ? 1 : -1;
 
-        /* Construct a list of all the elements in the given column. */
-        int nrow = AbstractStarTable
-                  .checkedLongToInt( dataModel_.getRowCount() );
-        Item[] items = new Item[ nrow ];
-        try ( RowSequence rseq = dataModel_.getRowSequence() ) {
-            int ir = 0;
-            while ( rseq.next() ) {
-                items[ ir ] =
-                    new Item( ir, (Comparable<?>) rseq.getCell( icol ) );
-                ir++;
+        /* Construct an array of all the sort keys. */
+        int nv = order.getExpressions().length;
+        int nrow =
+            AbstractStarTable.checkedLongToInt( dataModel_.getRowCount() );
+        TopcatJELRowReader rdr = TopcatJELRowReader.createAccessReader( this );
+        CompiledExpression[] compExs =
+            JELUtils.compileExpressions( rdr, order.getExpressions() );
+        RankedItem[] items = new RankedItem[ nrow ];
+        for ( int irow = 0; irow < nrow; irow++ ) {
+            Comparable<?>[] values = new Comparable<?>[ nv ];
+            for ( int iv = 0; iv < nv; iv++ ) {
+                Object value;
+                try {
+                    value = rdr.evaluateAtRow( compExs[ iv ], irow );
+                }
+                catch ( Throwable e ) {
+                    value = null;
+                }
+                assert value == null || value instanceof Comparable;
+                if ( value instanceof Comparable ) {
+                    values[ iv ] = (Comparable<?>) value;
+                }
             }
+            items[ irow ] = new RankedItem( values, irow );
         }
 
         /* Sort the list on the ordering of the items. */
-        Arrays.parallelSort( items );
+        Comparator<RankedItem> comparator = ( item1, item2 ) -> {
+            Comparable<?>[] vals1 = item1.values_;
+            Comparable<?>[] vals2 = item2.values_;
+            for ( int iv = 0; iv < nv; iv++ ) {
+                Comparable<?> v1 = vals1[ iv ];
+                Comparable<?> v2 = vals2[ iv ];
+                final int cmp;
+                if ( v1 != null && v2 != null ) {
+                    @SuppressWarnings("unchecked")
+                    int cmp0 = ((Comparable<Object>) v1).compareTo( v2 );
+                    cmp = cmp0;
+                }
+                else if ( v1 == null && v2 == null ) {
+                    cmp = 0;
+                }
+                else {
+                    cmp = v1 == null ? 1 : -1;
+                }
+                if ( cmp != 0 ) {
+                    return sense * cmp;
+                }
+            }
+            return 0;
+        };
+        Arrays.parallelSort( items, comparator );
 
         /* Construct and return a list of reordered ranks from the
          * sorted array. */
         int[] rowMap = new int[ nrow ];
         for ( int i = 0; i < nrow; i++ ) {
-            rowMap[ i ] = items[ i ].rank;
+            rowMap[ i ] = items[ i ].rank_;
         }
         return rowMap;
     }
@@ -1141,7 +1207,7 @@ public class TopcatModel {
                .setVisible( true );
             }
             else if ( this == unsortAct_ ) {
-                sortBy( SortOrder.NONE, false );
+                sortBy( SortOrder.NONE, true );
             }
             else {
                 assert false;
@@ -1149,134 +1215,6 @@ public class TopcatModel {
         }
     }
 
-    /**
-     * ButtonModel used for storing whether sorts should go up or down.
-     */
-    private class SortSenseModel extends JToggleButton.ToggleButtonModel {
-        boolean lastAscending_ = true;
-        public void setSelected( boolean ascending ) {
-            if ( ascending != lastAscending_ ) {
-
-                /* If the table view has a current (non-null) sort order, 
-                 * reverse it in place. */
-                int[] rowMap = viewModel_.getRowMap();
-                if ( rowMap != null ) {
-                    for ( int i = 0, j = rowMap.length - 1; i < j; i++, j-- ) {
-                        int c = rowMap[ i ];
-                        rowMap[ i ] = rowMap[ j ];
-                        rowMap[ j ] = c;
-                    }
-                    viewModel_.setRowMap( rowMap );
-                }
-
-                /* Store the changed state. */
-                lastAscending_ = ascending;
-                fireStateChanged();
-                fireModelChanged( TopcatEvent.CURRENT_ORDER, null );
-            }
-        }
-        public boolean isSelected() {
-            return lastAscending_;
-        }
-    }
-
-    /**
-     * ComboBoxModel used for storing the available and last-invoked
-     * sort orders.
-     */
-    private class SortSelectionModel implements ComboBoxModel<SortOrder> {
-
-        private final ColumnComboBoxModel sortColModel_;
-        private SortOrder lastSort_;
-
-        SortSelectionModel() {
-            sortColModel_ =
-                    new RestrictedColumnComboBoxModel( columnModel_, true ) {
-                public boolean acceptColumn( ColumnInfo cinfo ) {
-                    return Comparable.class
-                          .isAssignableFrom( cinfo.getContentClass() );
-                }
-            };
-            lastSort_ = SortOrder.NONE;
-        }
-
-        /**
-         * Turns a column identifier into a sort order definition.
-         */
-        public SortOrder getElementAt( int index ) {
-            return new SortOrder( sortColModel_.getElementAt( index ) );
-        }
-
-        public int getSize() {
-            return sortColModel_.getSize();
-        }
-
-        public void addListDataListener( ListDataListener l ) {
-            sortColModel_.addListDataListener( l );
-        }
-
-        public void removeListDataListener( ListDataListener l ) {
-            sortColModel_.removeListDataListener( l );
-        }
-
-        /**
-         * Returns the most recent selected sort. 
-         */
-        public Object getSelectedItem() {
-            return lastSort_;
-        }
-
-        /**
-         * Selecting an item in this model triggers the actual sort.
-         * All sorts pass through here.
-         */
-        public void setSelectedItem( Object item ) {
-            SortOrder order = (SortOrder) item;
-
-            /* Do nothing if the selected item is being set null - this
-             * corresponds to a JComboBox deselection, which happens 
-             * immediately prior to a selection when the control is
-             * activated. */
-            if ( order == null ) {
-                return;
-            }
-
-            /* Do nothing if the order is the same one we've just had. */
-            if ( order.equals( lastSort_ ) ) {
-                return;
-            }
-
-            /* OK do the sort, and install it in the viewModel. */
-            int[] rowMap;
-            if ( order.equals( SortOrder.NONE ) ) {
-                rowMap = null;
-            }
-            else {
-                TableColumn tcol = order.getColumn();
-                try {
-                    rowMap = getSortOrder( tcol.getModelIndex(), 
-                                           sortSenseModel_.isSelected() );
-                }
-                catch ( IOException e ) {
-                    Toolkit.getDefaultToolkit().beep();
-                    e.printStackTrace( System.err );
-                    setSelectedItem( SortOrder.NONE );
-                    return;
-                }
-            }
-
-            /* Install the new sorted order in the table view model. */
-            viewModel_.setOrder( rowMap );
-
-            /* Store the selected value. */
-            lastSort_ = order;
-
-            /* Inform TopcatListeners. */
-            fireModelChanged( TopcatEvent.CURRENT_ORDER, null );
-        }
-    }
-
-
     /**
      * ComboBoxModel used for storing the last-invoked subset selection.
      */
@@ -1355,4 +1293,16 @@ public class TopcatModel {
             }
         }
     }
+
+    /**
+     * Utility class that aggregates a rank and an array, used for sorting.
+     */
+    private static class RankedItem {
+        final Comparable<?>[] values_;
+        final int rank_;
+        RankedItem( Comparable<?>[] values, int rank ) {
+            values_ = values;
+            rank_ = rank; 
+        }
+    }
 }
