@@ -60,6 +60,7 @@ import javax.swing.event.ListSelectionListener;
 import uk.ac.starlink.topcat.AuxWindow;
 import uk.ac.starlink.topcat.BasicAction;
 import uk.ac.starlink.topcat.BasicCheckBoxList;
+import uk.ac.starlink.topcat.CheckBoxList;
 import uk.ac.starlink.topcat.LineBox;
 import uk.ac.starlink.topcat.MethodWindow;
 import uk.ac.starlink.topcat.ResourceIcon;
@@ -76,11 +77,10 @@ import uk.ac.starlink.util.Loader;
  * @author   Mark Taylor
  * @since    21 Dec 2017
  */
-@SuppressWarnings({"unchecked","rawtypes"})
 public class ActivationWindow extends AuxWindow {
 
     private final TopcatModel tcModel_;
-    private final DefaultListModel listModel_;
+    private final DefaultListModel<ActivationEntry> listModel_;
     private final BasicCheckBoxList<ActivationEntry> list_;
     private final JComponent configContainer_;
     private final JComponent outputContainer_;
@@ -128,7 +128,7 @@ public class ActivationWindow extends AuxWindow {
      * @param  tcModel  topcat model
      * @param  parent  parent window, may be used for positioning
      */
-    @SuppressWarnings("fallthrough")
+    @SuppressWarnings({"fallthrough","this-escape"})
     public ActivationWindow( final TopcatModel tcModel, Component parent ) {
         super( tcModel, "Activation Actions", parent );
         tcModel_ = tcModel;
@@ -143,41 +143,39 @@ public class ActivationWindow extends AuxWindow {
         final Color enabledFg = UIManager.getColor( "Label.foreground" );
         final Color disabledFg =
             UIManager.getColor( "Label.disabledForeground" );
-        list_ = new BasicCheckBoxList<ActivationEntry>( ActivationEntry.class,
-                                                        true ) {
-            @Override
-            protected void configureEntryRenderer( JComponent entryRenderer,
-                                                   ActivationEntry item,
-                                                   int index ) {
+        CheckBoxList.Rendering<ActivationEntry,JLabel> rendering =
+                new CheckBoxList.Rendering<ActivationEntry,JLabel>() {
+            public JLabel createRendererComponent() {
+                return new JLabel();
+            }
+            public void configureRendererComponent( JLabel label,
+                                                    ActivationEntry item,
+                                                    int index ) {
                 ActivationType atype = item.getType();
                 ActivatorConfigurator configurator = item.getConfigurator();
                 boolean isEnabled = configurator.getActivator() != null;
                 boolean isChecked = list_.isChecked( item );
-                if ( entryRenderer instanceof JLabel ) {
-                    JLabel label = (JLabel) entryRenderer;
-                    label.setText( atype.getName() );
-                    label.setEnabled( isEnabled );
-                    final Color fg;
-                    if ( isEnabled ) {
-                        fg = item.isBlocked() && isChecked ? WARNING_FG
-                                                           : enabledFg;
-                    }
-                    else {
-                        fg = disabledFg;
-                    }
-                    label.setForeground( fg );
+                label.setText( atype.getName() );
+                label.setEnabled( isEnabled );
+                final Color fg;
+                if ( isEnabled ) {
+                    fg = item.isBlocked() && isChecked ? WARNING_FG
+                                                       : enabledFg;
                 }
                 else {
-                    assert false;
+                    fg = disabledFg;
                 }
+                label.setForeground( fg );
             }
+        };
+        list_ = new BasicCheckBoxList<ActivationEntry>( true, rendering ) {
             @Override
             public void setChecked( ActivationEntry entry, boolean isChecked ) {
                 super.setChecked( entry, isChecked );
                 updateActivations();
             }
         };
-        listModel_ = (DefaultListModel) list_.getModel();
+        listModel_ = list_.getModel();
 
         list_.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
         JComponent listContainer = new JPanel( new BorderLayout() );
@@ -536,8 +534,7 @@ public class ActivationWindow extends AuxWindow {
         int nEnabled = 0;
         int nActive = 0;
         for ( int i = 0; i < listModel_.getSize(); i++ ) {
-            ActivationEntry entry =
-                (ActivationEntry) listModel_.getElementAt( i );
+            ActivationEntry entry = listModel_.getElementAt( i );
             if ( entry.getConfigurator().getActivator() != null ) {
                 nEnabled++;
                 if ( list_.isChecked( entry ) ) {
@@ -625,7 +622,7 @@ public class ActivationWindow extends AuxWindow {
             }
         }
         if ( selected == null && listModel_.getSize() == 0 ) {
-            selected = (ActivationEntry) listModel_.getElementAt( 0 );
+            selected = listModel_.getElementAt( 0 );
         }
         list_.setSelectedValue( selected, true );
         updateSelection();
@@ -661,7 +658,7 @@ public class ActivationWindow extends AuxWindow {
      * @param  lrow  new active row
      */
     private void updateActiveRow( long lrow ) {
-        tcModel_.fireModelChanged( TopcatEvent.ROW, new Long( lrow ) );
+        tcModel_.fireModelChanged( TopcatEvent.ROW, Long.valueOf( lrow ) );
     }
 
     /**
@@ -720,7 +717,8 @@ public class ActivationWindow extends AuxWindow {
      * @return   index of <code>entry</code> in <code>model</code>,
      *           or -1 if not found
      */
-    private int getEntryIndex( ListModel model, ActivationEntry entry ) {
+    private int getEntryIndex( ListModel<ActivationEntry> model,
+                               ActivationEntry entry ) {
         int n = model.getSize();
         for ( int i = 0; i < n; i++ ) {
             if ( model.getElementAt( i ) == entry ) {
@@ -899,6 +897,8 @@ public class ActivationWindow extends AuxWindow {
             NopActivationType.INSTANCE,
             new TopcatSkyPosActivationType(),
             new SendSkyPosActivationType(),
+            new ViewHips2fitsActivationType(),
+            new SendHips2fitsActivationType(),
             new InvokeDatalinkActivationType(),
             new ViewDatalinkActivationType(),
             new ViewImageActivationType(),
@@ -915,6 +915,7 @@ public class ActivationWindow extends AuxWindow {
             new DelayActivationType(),
             new JelActivationType(),
             new ShellActivationType(),
+            new SendCustomActivationType(),
             new SendIndexActivationType( true ),
         } ) );
         return list.toArray( new ActivationType[ 0 ] );
@@ -1405,7 +1406,9 @@ public class ActivationWindow extends AuxWindow {
             }
             try {
                 ActivationType atype =
-                    (ActivationType) Class.forName( clazzName ).newInstance();
+                    (ActivationType) Class.forName( clazzName )
+                                          .getDeclaredConstructor()
+                                          .newInstance();
                 typeMap_.put( atype.getClass().getName(), atype );
                 return atype;
             }
