@@ -391,6 +391,9 @@ public class TopcatSampControl {
             /* Load PDS4 table by reference. */
             new TableLoadHandler( "table.load.pds4", "pds4" ),
 
+            /* Load MOC as FITS file. */
+            new TableLoadHandler( "coverage.load.moc.fits", "fits" ),
+
             /* Load table with supplied format by reference. */
             new TableLoadHandler( TopcatSender.TOPCAT_LOAD_MTYPE, null ),
 
